from fast_trade import take_action, determine_action


def test_take_action_greater_than():
    mock_strategy = [["close", ">", "short"]]
    mock_row = [1523937963, 0.0212, 0.01, 0.025, 0.01, 36898, 0.0112]
    mock_columns = ["date", "close", "open", "high", "low", "volume", "short"]

    res = take_action(mock_row, mock_strategy, mock_columns)
    assert res == True


def test_take_action_less_than():
    mock_strategy = [["close", "<", "short"]]
    mock_row = [1523937963, 0.0212, 0.01, 0.025, 0.01, 36898, 0.0112]
    mock_columns = ["date", "close", "open", "high", "low", "volume", "short"]

    res = take_action(mock_row, mock_strategy, mock_columns)
    assert res == False


def test_take_action_equal():
    mock_strategy = [["close", "=", "short"]]
    mock_row = [1523937963, 0.0212, 0.01, 0.025, 0.01, 36898, 0.0212]
    mock_columns = ["date", "close", "open", "high", "low", "volume", "short"]

    res = take_action(mock_row, mock_strategy, mock_columns)
    assert res == True



import zipfile
import os
import csv
import json
import requests
import pandas as pd

# flake8: noqa
# this is a mess
def determine_ms_or_s(timestamp):
    if len(timestamp) == 10:
        return "s"

    return "ms"


def prep_ohlcv_from_zip(pair, base_csv_path=""):
    csv_filename = f"{pair}.csv"
    csv_path = f"{base_csv_path}/{csv_filename}"
    if not os.path.isfile(csv_path):
        # check for a zip
        if os.path.isfile("{}.zip".format(csv_path)):
            with zipfile.ZipFile("{}.zip".format(csv_path), "r") as zip_file:
                zip_file.extract(csv_filename, base_csv_path)

    return csv_path


def build_log_dataframe(log, symbol):
    csv_path = os.getenv("CSV_PATH")
    csv_filename = f"{csv_path}{symbol}.csv"
    csv_zipfilename = f"{csv_path}{symbol}.csv.zip"

    if os.path.isfile(csv_zipfilename):
        with zipfile.ZipFile(csv_zipfilename, "r") as zip_ref:
            zip_ref.extractall(f"{symbol}.csv")

    # print(csv_filename)


if __name__ == "__main__":
    csv_path = "/Users/jedmeier/2017_standard"
    res = prep_ohlcv_from_zip("BTCUSDT", csv_path)
    print("res: ", res)



def analyze_df(df, strategy):
    in_trade = False
    column_map = list(df.columns)
    close_column_idx = column_map.index("close")
    action_col_idx = column_map.index("actions")

    base_balance = strategy["base_balance"]

    aux_balance = 0.0

    aux_log = []
    base_log = []

    smooth_base = []
    for idx, row in enumerate(df.values):
        close = row[close_column_idx]
        if row[action_col_idx] == "e" and not in_trade:
            aux_balance = enter_trade(close, base_balance)

            tmp_base_balance = exit_trade(close, aux_balance)
            base_balance = 0
            in_trade = True

        if row[action_col_idx] == "x" and in_trade:
            base_balance = exit_trade(close, aux_balance)
            aux_balance = 0
            in_trade = False

        if base_balance:
            smooth_base.append(base_balance)
        else:
            smooth_base.append(tmp_base_balance)

        aux_log.append(aux_balance)
        base_log.append(base_balance)

    if in_trade and strategy["exit_on_end"]:
        close = df.values[-1][close_column_idx]
        base_balance = exit_trade(close, aux_balance)
        smooth_base.append(base_balance)
        aux_balance = 0
        aux_log.append(aux_balance)
        base_log.append(base_balance)

    return aux_log, base_log, smooth_base


def enter_trade(close, base_balance):
    """ returns new aux balance """
    if base_balance:
        return round(base_balance / close, 8)
    return 0.0


def exit_trade(close, aux_balance):
    """ returns new base balance """
    if aux_balance:
        return round(aux_balance * close, 8)
    return 0.0



import json
import os
import datetime
import pandas as pd
import matplotlib.pyplot as plt


def open_strat_file(fp):

    try:
        with open(fp, "r") as json_file:
            strat = json.load(json_file)

        return strat
    except Exception as e:
        print("error: ", e)


def format_command(command):

    command_obj = help_dict[command]
    ret_str = f"\t{command}"
    ret_str += f"\n\t\t\t{command_obj['description']}"

    ret_str += f"\n\t\tusage"
    for ex in command_obj["examples"]:
        ret_str += f"\n\t\t\t{ex}"

    ret_str += f"\n\t\targs"
    for arg in command_obj["args"].keys():
        ret_str += f"\n\t\t\t{arg},"
        if command_obj["args"][arg]["required"]:
            ret_str += " required,"

        ret_str += f" {command_obj['args'][arg]['desc']}"

    return ret_str


def format_all_help_text():

    formated_string = "\nFast Trade Help\n\nCommands\n\n"

    for command in help_dict.keys():
        formated_string = formated_string + format_command(command)

    return formated_string


def create_plot(df):
    plot_df = pd.DataFrame(
        data={
            "Date": df.index,
            "Portfolio_Value": df["smooth_base"],
            "Close": df["close"],
        }
    )

    plot_df.plot(x="Date", y=["Portfolio_Value", "Close"])

    return plot_df


def save(result, strat_obj):
    """
    Save the dataframe, strategy, and plot into the specified path
    """
    save_path = "./saved_backtests"
    if not os.path.exists(save_path):
        os.mkdir(save_path)

    # dir exists, now make a new dir with the files
    new_dir = datetime.datetime.strftime(datetime.datetime.now(), "%Y_%m_%d_%H_%M_%S")

    new_save_dir = f"{save_path}/{new_dir}"
    os.mkdir(new_save_dir)

    # save the strat args
    with open(f"{new_save_dir}/strategy.json", "w") as summary_file:
        summary_file.write(json.dumps(strat_obj, indent=2))

    # summary file
    with open(f"{new_save_dir}/summary.json", "w") as summary_file:
        summary_file.write(json.dumps(result["summary"], indent=2))

    # dataframe
    result["df"].to_csv(f"{new_save_dir}/dataframe.csv")

    # plot
    create_plot(result["df"])
    plt.savefig(f"{new_save_dir}/plot.png")


help_dict = {
    "backtest": {
        "description": "Runs a backtest with the given parameters. Any strat modifications can be passed at the end of the command.",
        "examples": ["python -m fast_trade backtest --csv=./datafile.csv/ --strat=./strat.json"],
        "args": {
                "--csv": {
                    "desc": "path or paths to csv_file seperated by commands",
                    "required": True
                },
                "--strat": {
                    "desc": "path to strategy file, must json format",
                    "required": True
                },
                "--plot": {
                        "desc": "opens a basic plot using matlib.plot",
                        "required": False
                },
                "--save": {
                    "desc": "saves the dataframe, strategy, and plot in to the path",
                    "required": False
                }
        }
    }
}


import pytest
import pandas as pd
import random
from fast_trade import enter_trade, exit_trade, analyze_df


def test_enter_trade_1():
    mock_close = 10
    mock_aux_balance = 100
    res = enter_trade(mock_close, mock_aux_balance)

    assert res == 10


def test_enter_trade_2():
    mock_close = 0.025
    mock_aux_balance = 60

    res = enter_trade(mock_close, mock_aux_balance)
    assert res == 2400.0


def test_enter_trade_3():
    mock_close = 0.3123
    mock_aux_balance = 212.2333

    res = enter_trade(mock_close, mock_aux_balance)
    assert res == 679.58149215


def test_enter_trade_4():
    mock_close = 0.3123
    mock_aux_balance = 0

    res = enter_trade(mock_close, mock_aux_balance)
    assert res == 0


def test_exit_trade_1():
    mock_close = 0.99992
    mock_base_balance = 133.22

    aux = exit_trade(mock_close, mock_base_balance)

    assert aux == 133.2093424

def test_exit_trade_2():
    mock_close = 0.99992
    mock_base_balance = 0

    aux = exit_trade(mock_close, mock_base_balance)

    assert aux == 0.0



def test_analyze_df_1():
    mock_df = pd.read_csv("./test/ohlcv_data.csv.txt")

    mock_strategy = {
        "base_balance": 1000,
        "exit_on_end": True
    }

    mock_df['actions'] = ["e","h","x","x","x","e","x","h","h"]
    
    aux_log, base_log, smooth_base = analyze_df(mock_df, mock_strategy)

    assert aux_log == [100000.0, 100000.0, 0, 0, 0, 103237.41007194, 0, 0, 0]
    assert base_log == [0, 0, 2296.0, 2296.0, 2296.0, 0, 2274.32014388, 2274.32014388, 2274.32014388]
    assert smooth_base == [1000.0, 1000.0, 2296.0, 2296.0, 2296.0, 2296.0, 2274.32014388, 2274.32014388, 2274.32014388]


def test_analyze_df_2():
    mock_df = pd.read_csv("./test/ohlcv_data.csv.txt")

    mock_strategy = {
        "base_balance": 1000,
        "exit_on_end": True
    }
    mock_df['actions'] = ["e","h","h","x","h","h","e","h","h"]

    aux_log, base_log, smooth_base = analyze_df(mock_df, mock_strategy)

    assert aux_log == [100000.0, 100000.0, 100000.0, 0, 0, 0, 96232.41034952, 96232.41034952, 96232.41034952, 0]
    assert base_log == [0, 0, 0, 2120.0, 2120.0, 2120.0, 0, 0, 0, 2065.1475261]
    assert smooth_base == [1000.0, 1000.0, 1000.0, 2120.0, 2120.0, 2120.0, 2120.0, 2120.0, 2120.0, 2065.1475261]
